#!/usr/bin/env node

'use strict';

require('chromedriver');

var execSync = require('child_process').execSync,
    expect = require('expect.js'),
    path = require('path'),
    superagent = require('superagent'),
    webdriver = require('selenium-webdriver');

var by = webdriver.By,
    Keys = webdriver.Key,
    until = webdriver.until,
    Builder = require('selenium-webdriver').Builder;

var username = 'admin',
    password = 'changeme';

process.env.NODE_TLS_REJECT_UNAUTHORIZED = '0';


describe('Application life cycle test', function () {
    this.timeout(0);

    var server, browser = new Builder().forBrowser('chrome').build();

    before(function (done) {
        var seleniumJar= require('selenium-server-standalone-jar');
        var SeleniumServer = require('selenium-webdriver/remote').SeleniumServer;
        server = new SeleniumServer(seleniumJar.path, { port: 4444 });
        server.start();

        done();
    });

    after(function (done) {
        browser.quit();
        server.stop();
        done();
    });

    var LOCATION = 'test';
    var TEST_TIMEOUT = 10000;
    var app;

    function exists(selector) {
        return browser.wait(until.elementLocated(selector), TEST_TIMEOUT);
    }

    function visible(selector, callback) {
        return exists(selector).then(function () {
            return browser.wait(until.elementIsVisible(browser.findElement(selector)), TEST_TIMEOUT);
        });
    }

    function baseUrl() {
        if (app.manifest.version === '1.4.0') return `https://${app.fqdn}/p`;
        return `https://${app.fqdn}`;
    }

    // we do this because it perm redirects to /p/ in old versions
    function clearCache() {
        // https://stackoverflow.com/questions/49614217/selenium-clear-chrome-cache
        // defaut clearance is 1 hour of cache
        return browser.get('chrome://settings/clearBrowserData').then(function () {
            return visible(by.css('* /deep/ #clearBrowsingDataConfirm'));
        }).then(function () {
            return browser.findElement(by.css('* /deep/ #clearBrowsingDataConfirm')).click();
        }).then(function () {
            return browser.sleep(5000);
        });
    }

    function login(password, callback) {
        browser.get('https://' + app.fqdn).then(function () {
            return visible(by.id('loginButton'));
        }).then(function () {
            return browser.findElement(by.id('username')).sendKeys(username);
        }).then(function () {
            return browser.findElement(by.id('passwordPlain')).sendKeys(password);
        }).then(function () {
            return browser.findElement(by.id('loginButton')).click();
        }).then(function () {
            return browser.wait(until.elementLocated(by.id('dropdown-configure')), TEST_TIMEOUT);
        }).then(function () {
            callback();
        });
    }

    function logout(callback) {
        var logout_btn = by.xpath('//ul[@class="dropdown-menu"]/li/a[@class="signout"]');

        browser.get('https://' + app.fqdn).then(function () {
            return visible(by.id('stream'));
        }).then(function () {
            return browser.findElement(by.className('dropdown-toggle')).click();
        }).then(function () {
            return visible(logout_btn);
        }).then(function () {
            return browser.findElement(logout_btn).click();
        }).then(function () {
            return browser.wait(until.elementLocated(by.id('loginButton')), TEST_TIMEOUT);
        }).then(function () {
            callback();
        });
    }

    function addSubscription(callback) {
        var url = "https://cloudron.io/blog/rss.xml";

        browser.get(`${baseUrl()}/i/?c=subscription`).then(function () {
            return visible(by.xpath('//input[@name="url_rss"]'));
        }).then(function () {
            return browser.findElement(by.xpath('//input[@name="url_rss"]')).sendKeys(url);
        }).then(function () {
            return browser.findElement(by.xpath('//form[@id="add_rss"]/div/button[@type="submit"]')).click();
        }).then(function () {
            return visible(by.xpath('//div[@id="notification" and @class="notification good"]'));
        }).then(function () {
            callback();
        });
    }

    function addUser(password, callback) {
        var test_username = 'test';

        browser.get(`${baseUrl()}/i/?c=user&a=manage`).then(function () {
            return visible(by.id('new_user_name'));
        }).then(function () {
            return browser.findElement(by.id('new_user_name')).sendKeys(test_username);
        }).then(function () {
            return browser.findElement(by.id('new_user_passwordPlain')).sendKeys(password);
        }).then(function () {
            return browser.findElement(by.xpath('//button[text()="Create"]')).click();
        }).then(function () {
            return visible(by.xpath('//div[@id="notification" and @class="notification good"]'));
        }).then(function () {
            callback();
        });
    }

    function enableApi(callback) {
        browser.get(`${baseUrl()}/i/?c=auth`).then(function () {
            return browser.findElement(by.id('api_enabled')).click();
        }).then(function () {
            return browser.findElement(by.xpath('//button[text()="Submit"]')).submit();
        }).then(function () {
            callback();
        });
    }

    function checkApiConfiguration(callback) {
        browser.get(`${baseUrl()}/api/`).then(function () {
            return exists(by.xpath('//dd[@id="greaderOutput" and contains(text(), "PASS")]'));
        }).then(function () {
            return exists(by.xpath('//dd[@id="feverOutput" and contains(text(), "PASS")]'));
        }).then(function () {
            callback();
        });
    }

    function getAppInfo() {
        var inspect = JSON.parse(execSync('cloudron inspect'));
        app = inspect.apps.filter(function (a) { return a.location === LOCATION || a.location === LOCATION + '2'; })[0];
        expect(app).to.be.an('object');
    }

    function subscriptionExists(callback) {
        browser.get(`${baseUrl()}/i/?get=c_1`).then(function () {
            return visible(by.xpath('//a[text()="Cloudron.io"]'));
        }).then(function () {
            callback();
        });
    }

    function getStaticExtensionFile(callback) {
        superagent.get(`${baseUrl()}/ext.php?f=xExtension-StickyFeeds/static/script.js&t=js`)
            .buffer(true)
            .end(function (err, res) {
                expect(err).to.be(null);
                expect(res.status).to.be(200);
                expect(res.text).to.contain('sticky_feeds'); // relies on the buffer flag above
                callback();
        });
    }

    xit('build app', function () {
        execSync('cloudron build', { cwd: path.resolve(__dirname, '..'), stdio: 'inherit' });
    });

    it('install app', function () {
        execSync('cloudron install --location ' + LOCATION, { cwd: path.resolve(__dirname, '..'), stdio: 'inherit' });
    });

    it('can get app information', getAppInfo);

    it('can login', login.bind(null, password));
    it('can subscribe', addSubscription);
    it('can add users', addUser.bind(null, password));
    it('can enable API', enableApi);
    it('can check configuration', checkApiConfiguration);
    it('subscription exists', subscriptionExists);
    it('can get static extension file', getStaticExtensionFile);
    it('can logout', logout);

    it('backup app', function () {
        execSync('cloudron backup create --app ' + app.id, { cwd: path.resolve(__dirname, '..'), stdio: 'inherit' });
    });

    it('restore app', function () {
        const backups = JSON.parse(execSync('cloudron backup list --raw'));
        execSync('cloudron uninstall --app ' + app.id, { cwd: path.resolve(__dirname, '..'), stdio: 'inherit' });
        execSync('cloudron install --location ' + LOCATION, { cwd: path.resolve(__dirname, '..'), stdio: 'inherit' });
        getAppInfo();
        execSync(`cloudron restore --backup ${backups[0].id} --app ${app.id}`, { cwd: path.resolve(__dirname, '..'), stdio: 'inherit' });
    });

    it('can login', login.bind(null, password));
    it('can check configuration', checkApiConfiguration);
    it('subscription exists', subscriptionExists);
    it('can get static extension file', getStaticExtensionFile);
    it('can logout', logout);

    it('move to different location', function () {
        browser.manage().deleteAllCookies();
        execSync('cloudron configure --location ' + LOCATION + '2 --app ' + app.id, { cwd: path.resolve(__dirname, '..'), stdio: 'inherit' });
        var inspect = JSON.parse(execSync('cloudron inspect'));
        app = inspect.apps.filter(function (a) { return a.location === LOCATION + '2'; })[0];
        expect(app).to.be.an('object');
    });

    it('can login', login.bind(null, password));
    it('can check configuration', checkApiConfiguration);
    it('subscription exists', subscriptionExists);
    it('can get static extension file', getStaticExtensionFile);
    it('can logout', logout);

    it('uninstall app', function () {
        execSync('cloudron uninstall --app ' + app.id, { cwd: path.resolve(__dirname, '..'), stdio: 'inherit' });
    });

    // test update
    it('can install app', function () {
        execSync('cloudron install --appstore-id org.freshrss.cloudronapp --location ' + LOCATION, { cwd: path.resolve(__dirname, '..'), stdio: 'inherit' });
    });

    it('can get app information', getAppInfo);
    it('can login', login.bind(null, password));
    it('can subscribe', addSubscription);
    it('can add users', addUser.bind(null, password));

    it('can update', function () {
        execSync('cloudron update --app ' + app.id, { cwd: path.resolve(__dirname, '..'), stdio: 'inherit' });
        var inspect = JSON.parse(execSync('cloudron inspect'));
        app = inspect.apps.filter(function (a) { return a.location === LOCATION; })[0];
        expect(app).to.be.an('object');
    });

    it('can login', login.bind(null, password));
    it('subscription exists', subscriptionExists);
    it('can get static extension file', getStaticExtensionFile);

    it('uninstall app', function () {
        execSync('cloudron uninstall --app ' + app.id, { cwd: path.resolve(__dirname, '..'), stdio: 'inherit' });
    });
});
